import pytest

from crudlfap.utils import view_reverse
from crudlfap.views.generic import View


class ExampleView(View):
    pass


class DerivedView(ExampleView):
    pass


def test_example_view():
    assert ExampleView.get_slug() == 'example'
    assert ExampleView.get_url_pattern() == 'example/$'


def test_derived_view():
    assert DerivedView.get_slug() == 'derived'
    assert DerivedView.get_url_pattern() == 'derived/$'


urlpatterns = [
    ExampleView.as_url(),
    ExampleView.as_url(slug='example2'),
]


@pytest.mark.urls('crudlfap.views.test_generic')
def test_view_with_slug():
    view = view_reverse('example')
    assert view.get_slug() == 'example'
    assert view.__name__ == ExampleView.__name__
    assert view.get_url_pattern() == 'example/$'
    assert view.get_app_label() == 'crudlfap'
'''
@pytest.mark.urls('crudlfap.views.test_generic')
def test_view_with_slug():
    view = view_reverse('example2')
    assert view == ExampleView
'''



from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Migrate/Createsuperuser/Runserver'

    def handle(self, *args, **options):
        call_command('migrate')

        from django.conf import settings
        from django.apps import apps
        user_model = apps.get_model(settings.AUTH_USER_MODEL)
        if not user_model.objects.count():
            call_command('createsuperuser')

        call_command('runserver')



from crudlfap import crudlfap

from .models import Song


class SongRouter(crudlfap.Router):
    fields = '__all__'
    icon = 'music'

    views = [
        'crudlfap.views.generic.DeleteView',
        'crudlfap.views.generic.UpdateView',
        'crudlfap.views.generic.DetailView',
        'crudlfap.views.generic.CreateView',
        dict(
            _cls='crudlfap_filtertables2.views.FilterTables2ListView',
            filter_fields=['artist', 'name'],
        )
    ]

urlpatterns = SongRouter(Song).urlpatterns()



def base(request):
    c = dict(base='base.html')
    base = request.POST.get('base', request.GET.get('base', False))

    if base in ('ajax', 'modal'):
        c['base'] = 'base_{}.html'.format(base)

    return c



#!/usr/bin/env python
import os
import sys


def main():
    os.environ.setdefault(
        "DJANGO_SETTINGS_MODULE",
        "crudlfap_example.settings"
    )
    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-19 18:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('song', '0002_add_duration'),
    ]

    operations = [
        migrations.AddField(
            model_name='song',
            name='upload',
            field=models.FileField(default='', upload_to='songs'),
            preserve_default=False,
        ),
    ]



import pytest

from django.db import models

from .jinja2 import get_view
from .routers import Router
from .views.generic import View


class Artist(models.Model):
    name = models.CharField(max_length=200)

    class Meta:
        managed = False


class ExampleView(View):
    pass


test_urls = [
    Router(Artist).urlpatterns(),
    ExampleView.as_url(),
]


@pytest.mark.urls('crudlfap.test_utils.test_urls')
def test_get_view_by_url_name():
    pass



"""Import everything we expose in crudlfap namespace."""

from .apps import _installed

from .routers import Router

from .views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    FormView,
    ListView,
    ModelViewMixin,
    ObjectFormView,
    ObjectViewMixin,
    UpdateView,
    View,
    ViewMixin,
)

if _installed('crudlfap_filtertables2'):
    from crudlfap_filtertables2.views import FilterTables2ListView



from django.conf import settings
from django.conf.urls import include, url
from django.views import generic

urlpatterns = [
    url(r'^artist/', include('crudlfap_example.artist.urls')),
    url(r'^song/', include('crudlfap_example.song.urls')),

    url(r'^auth/', include('django.contrib.auth.urls')),
    url(r'^debug/', include('crudlfap.urls')),
    url(r'^$', generic.TemplateView.as_view(
        template_name='crudlfap/home.html')),
]

if 'debug_toolbar' in settings.INSTALLED_APPS and settings.DEBUG:
    import debug_toolbar
    urlpatterns += [
        url(r'^__debug__/', include(debug_toolbar.urls)),
    ]



"""Django AppConfig for the crudlfap module."""

from django import apps
from django.conf import settings

from .views import generic


def _installed(*apps):
    for app in apps:
        if app not in settings.INSTALLED_APPS:
            return False
    return True


class DefaultConfig(apps.AppConfig):
    """Default AppConfig."""

    name = 'crudlfap'

    def get_default_views(self):  # pylint: disable=no-self-use
        """Return the default views to use in a new router."""
        views = [
            'crudlfap.views.generic.DeleteView',
            'crudlfap.views.generic.UpdateView',
            'crudlfap.views.generic.DetailView',
            'crudlfap.views.generic.CreateView',
        ]

        if _installed('crudlfap_filtertables2'):
            from crudlfap_filtertables2.views import FilterTables2ListView
            views.append(FilterTables2ListView)
        else:
            views.append(generic.ListView)

        return views



import collections

from crudlfap.views.generic import ListView

from django import template
from django.db import models
from django.utils.translation import ugettext_lazy as _

import django_filters
from django_filters.views import FilterView

import django_tables2 as tables
from django_tables2.views import SingleTableMixin


class Table(tables.Table):
    def render_crudlfap(self, record):
        from django.template import loader
        from crudlfap.routers import Router
        context = dict(
            object=record,
            views=Router.registry[type(record)].get_menu('object'),
        )
        template = loader.select_template([
            '{}/_{}_actions.html'.format(
                type(record)._meta.app_label,
                type(record)._meta.model_name,
                ),
            'crudlfap/_actions.html',
            ])
        return template.render(context)

    def render_tags(self, record):
        html = []
        for tag in record.tags.all():
            html.append(unicode(tag))

        return ' '.join(html)


class FilterTables2ListView(SingleTableMixin, FilterView, ListView):
    urlre = r'$'
    default_template_name = 'crudlfap_filtertables2/list.html'
    template_name_suffix = '_list'
    icon = 'fa fa-fw fa-table'
    slug = 'list'
    table_class = Table

    def dispatch(self, *a, **k):
        if not getattr(self, 'filter_fields', None):
            self.filter_fields = self.fields
        if not getattr(self, 'table_fields', None):
            self.table_fields = self.fields
        return super().dispatch(*a, **k)

    def get_context_data(self, *args, **kwargs):
        c = super().get_context_data(*args, **kwargs)
        c['action_views'] = self.router.get_menu('object')
        return c

    def get_table_class(self):
        list_display_links = getattr(self, 'list_display_links', None)
        if not list_display_links:
            for field in self.table_fields:
                model_field = self.model._meta.get_field(field)
                if isinstance(model_field, models.CharField):
                    list_display_links = [field]
                    break

        attrs = collections.OrderedDict(
                Meta=type(
                    'Meta',
                    (object,),
                    dict(
                        model=self.model,
                        fields=self.fields,
                        ),
                    ),
                )

        attrs.update({
            i: tables.LinkColumn() for i in list_display_links
        })

        attrs.update(dict(
            crudlfap=tables.TemplateColumn(
                template_name='crudlfap/_actions.html',
                verbose_name=_('Actions'),
            ),
        ))

        return type(
            '{}Table'.format(self.model.__name__),
            (self.table_class,),
            attrs
        )

    def get_table_data(self):
        return self.filterset.qs

    def get_filterset_class(self):
        return type(
            '{}FilterSet'.format(self.model.__name__),
            (django_filters.FilterSet,),
            dict(
                Meta=getattr(self, 'filterset_meta', type(
                    'Meta',
                    (object,),
                    dict(
                        model=self.model,
                        fields=self.filter_fields,
                        filter_overrides={
                            models.CharField: {
                               'filter_class': django_filters.CharFilter,
                               'extra': lambda f: {
                                   'lookup_expr': 'icontains',
                               },
                           },
                        }
                    )
                ))
            )
        )



from django import http
from django.conf import settings
from django.utils.module_loading import import_string
from django.views import generic


class UrlsView(generic.TemplateView):
    template_name = 'crudlfap/debug_urls.html'

    def get(self, request, *a, **k):
        if not request.user.is_superuser:
            return http.HttpResponse404()
        return super(UrlsView, self).get(request, *a, **k)

    def get_context_data(self):
        c = super(UrlsView, self).get_context_data()
        c['urlpatterns'] = urlpatterns = import_string(
            settings.ROOT_URLCONF
        ).urlpatterns
        for u in urlpatterns:
            print(u.regex.pattern, getattr(u, 'url_patterns', None))
        return c



"""
Crudlfa+ generic views and mixins.

Crudlfa+ takes views further than Django and are expected to:

- generate their URL definitions and reversions,
- check if a user has permission for an object,
- declare the names of the navigation menus they belong to.
"""
from django.conf.urls import url
from django.contrib import messages
from django.core.exceptions import FieldDoesNotExist
from django.utils.translation import ugettext_lazy as _
from django.views import generic


class ViewMixin(object):
    """
    Mixin to make crudlfa+ love a view.

    .. py:attribute:: slug

        Slug name of this view, often properly automatically generated
        from view class name uppon registration.

    .. py:attribute:: verbose_slug

        Verbose slug of the view for display.

    .. py:attribute:: url_pattern

        URL pattern to use for this view.
    """

    slug = None
    url_pattern = None
    style = 'default'
    fa_icon = 'question'
    material_icon = 'priority high'

    def get_template_names(self):
        """Give a chance to default_template_name."""
        template_names = super().get_template_names()
        default_template_name = getattr(self, 'default_template_name', None)
        if default_template_name:
            template_names.append(default_template_name)
        return template_names

    @classmethod
    def get_fa_icon(cls):
        return (
            getattr(cls, 'fa_icon', None) or
            getattr(cls.router, 'fa_icon', '')
        )

    @classmethod
    def as_url(cls, **kwargs):
        """Return the Django url object."""
        return url(
            '{}{}'.format(
                cls.router.prefix or '',
                cls.get_url_pattern(),
            ),
            cls.as_view(**kwargs),
            name=cls.get_url_name(),
        )

    @classmethod
    def get_url_pattern(cls):
        """Return the url pattern for this view."""
        if cls.url_pattern:
            return cls.url_pattern.format(cls.slug)
        return '{}/$'.format(cls.slug)

    @classmethod
    def get_url_name(cls):
        """Return the url name for this view which has a router."""
        return '{}_{}'.format(
            cls.router.model_name,
            cls.slug,
        )

    @classmethod
    def get_url_args(cls, *args):  # pylint: disable=unused-argument
        """Return url reverse args given these args."""
        return args

    @classmethod
    def reverse(cls, *args):
        """Reverse a url to this view with the given args."""
        from django.core.urlresolvers import reverse_lazy
        return reverse_lazy(
            cls.get_url_name(),
            args=cls.get_url_args(*args)
        )

    def get_title_html(self):
        """Return text for HTML title tag."""
        return self.title

    def get_title_heading(self):
        """Return text for page heading."""
        return self.title


class View(ViewMixin, generic.View):
    """Base view for crudlfap+."""


class ModelViewMixin(ViewMixin):
    """Mixin for views using a Model class but no instance."""

    menus = ['model']

    @property
    def title(self):
        return '{} {}'.format(
            _(self.slug),
            self.model._meta.verbose_name_plural,
        ).capitalize()

    @property
    def fields(self):
        """Return router.fields or None, field names if ``__all__``."""
        fields = getattr(self.router, 'fields', None)
        if fields == '__all__':
            fields = [
                f.name for f in self.model._meta.fields
                if not f.primary_key or not getattr(
                    self, 'with_pk', False)
            ]
        return fields

    @property
    def exclude(self):
        """Return router.exclude or None, field names if ``__all__``.."""
        return getattr(self.router, 'ecxlude', None)


class ObjectViewMixin(ModelViewMixin):
    """Mixin for views using a Model instance."""

    menus = ['object']

    @classmethod
    def get_url_args(cls, *args):
        if '<slug>' in cls.get_url_pattern():
            return [args[0].slug]
        return [args[0].pk]

    @classmethod
    def get_url_pattern(cls):
        """Identify the object by slug or pk in the pattern."""
        if cls.url_pattern:
            return cls.url_pattern.format(cls.slug)

        try:
            cls.model._meta.get_field('slug')
        except FieldDoesNotExist:
            return r'(?P<pk>\d+)/{}/$'.format(cls.slug)
        else:
            return r'(?P<slug>[\w\d_-]+)/{}/$'.format(cls.slug)

    @property
    def title(self):
        return '{} {} "{}"'.format(
            _(self.slug),
            self.model._meta.verbose_name,
            self.object
        ).capitalize()


class FormViewMixin(ViewMixin):
    """Mixin for views which have a Form."""
    success_url_next = True

    def get_success_url(self):
        url = super().get_success_url()
        if self.success_url_next and '_next' in self.request.POST:
            url = self.request.POST['_next']
        return url


class FormView(FormViewMixin, generic.FormView):
    """Base FormView class."""

    style = 'warning'
    default_template_name = 'crudlfap/form.html'


class ModelFormViewMixin(ModelViewMixin, FormViewMixin):
    """ModelForm ViewMixin using readable"""

    def form_invalid(self, form):
        messages.error(
            self.request,
            _(
                '{} {} error'.format(
                    self.slug,
                    self.model._meta.verbose_name
                ).capitalize()
            )
        )
        return super().form_invalid(form)

    def form_valid(self, form):
        messages.success(
            self.request,
            _(
                '%s %s: {}' % (self.slug, self.model._meta.verbose_name)
            ).format(form.instance).capitalize()
        )
        return super().form_valid(form)


class ObjectFormViewMixin(ObjectViewMixin, ModelFormViewMixin):
    """Custom form view mixin on an object."""


class ObjectFormView(ObjectFormViewMixin, generic.FormView):
    """Custom form view on an object."""


class CreateView(ModelFormViewMixin, generic.CreateView):
    """View to create a model object."""

    style = 'success'
    fa_icon = 'plus'
    material_icon = 'add'
    default_template_name = 'crudlfap/create.html'
    target = 'modal'


class DeleteView(ObjectFormViewMixin, generic.DeleteView):
    """View to delete a model object."""

    default_template_name = 'crudlfap/delete.html'
    style = 'danger'
    fa_icon = 'trash'
    material_icon = 'delete'
    target = 'modal'
    success_url_next = True

    def get_success_url(self):
        messages.success(
            self.request,
            _(
                '%s %s: {}' % (self.slug, self.model._meta.verbose_name)
            ).format(self.object).capitalize()
        )
        return self.router['list'].reverse()


class DetailView(ObjectViewMixin, generic.DetailView):
    """Templated model object detail view which takes a field option."""

    fa_icon = 'search-plus'
    material_icon = 'search'
    default_template_name = 'crudlfap/detail.html'

    @property
    def title(self):
        return str(self.object)

    def get_context_data(self, *a, **k):
        c = super(DetailView, self).get_context_data(*a, **k)
        c['fields'] = [
            {
                'field': self.model._meta.get_field(field),
                'value': getattr(self.object, field)
            }
            for field in self.fields
        ]
        return c

    @classmethod
    def get_url_pattern(cls):
        """Identify the object by slug or pk in the pattern."""
        if cls.url_pattern:
            return cls.url_pattern.format(cls.slug)

        try:
            cls.model._meta.get_field('slug')
        except FieldDoesNotExist:
            return r'(?P<pk>\d+)/$'
        else:
            return r'(?P<slug>[\w\d_-]+)/$'


class ListView(ModelViewMixin, generic.ListView):
    """Model list view."""

    default_template_name = 'crudlfap/list.html'
    url_pattern = '$'
    paginate_by = 10
    fa_icon = 'table'
    material_icon = 'list'
    menus = ['main']


class UpdateView(ObjectFormViewMixin, generic.UpdateView):
    """Model update view."""

    fa_icon = 'edit'
    material_icon = 'edit'
    default_template_name = 'crudlfap/update.html'
    target = 'modal'



from django.urls.base import get_resolver, get_urlconf
from django.core.urlresolvers import reverse


def view_reverse(url_name, urlconf=None):
    """Return the View class for a url name."""
    urlconf = urlconf or get_urlconf()
    resolver = get_resolver(urlconf)

    for url_pattern in resolver.url_patterns:
        if url_pattern.name == url_name:
            return url_pattern.callback.view_class



import re

from crudlfap import crudlfap

from django.db import models

import pytest


class Artist(models.Model):
    name = models.CharField(max_length=100)
    slug = models.CharField(max_length=100)

    class Meta:
        managed = False
        app_label = 'crudlfap_test'


class ArtistPicture(models.Model):
    path = models.ImageField(upload_to='lol')

    class Meta:
        managed = False
        app_label = 'crudlfap_test'
        verbose_name = 'Artist photo'


def assert_url_like(view, name, pattern):
    url = view.as_url()
    assert url.name == name
    assert url.regex == re.compile(pattern)


@pytest.fixture
def router():
    return crudlfap.Router(Artist)


def test_router_init():
    result = crudlfap.Router(Artist)
    assert result.model == Artist
    assert result.prefix == ''


def test_router_create(router):
    assert_url_like(
        router['create'],
        'artist_create',
        'create/$'
    )


def test_router_detail(router):
    assert_url_like(
        router['detail'],
        'artist_detail',
        '(?P<slug>[\w\d_-]+)/$'
    )


def test_router_delete(router):
    assert_url_like(
        router['delete'],
        'artist_delete',
        '(?P<slug>[\w\d_-]+)/delete/$'
    )


def test_router_list(router):
    assert_url_like(
        router['list'],
        'artist_list',
        '$'
    )


def test_router_update(router):
    assert_url_like(
        router['update'],
        'artist_update',
        '(?P<slug>[\w\d_-]+)/update/$'
    )


@pytest.fixture
def router_prefix():
    return crudlfap.Router(ArtistPicture, prefix='photo/')


def test_router_prefix(router_prefix):
    assert router_prefix.model == ArtistPicture
    assert router_prefix.prefix == 'photo/'


def test_router_prefix_create(router_prefix):
    assert_url_like(
        router_prefix['create'],
        'artistpicture_create',
        'photo/create/$'
    )


def test_router_prefix_detail(router_prefix):
    assert_url_like(
        router_prefix['detail'],
        'artistpicture_detail',
        'photo/(?P<pk>\d+)/$'
    )


def test_router_prefix_delete(router_prefix):
    assert_url_like(
        router_prefix['delete'],
        'artistpicture_delete',
        'photo/(?P<pk>\d+)/delete/$'
    )


def test_router_prefix_list(router_prefix):
    assert_url_like(
        router_prefix['list'],
        'artistpicture_list',
        'photo/$'
    )


def test_router_prefix_update(router_prefix):
    assert_url_like(
        router_prefix['update'],
        'artistpicture_update',
        'photo/(?P<pk>\d+)/update/$'
    )



from setuptools import setup, find_packages
import os


# Utility function to read the README file.
# Used for the long_description. It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='crudlfap',
    version='0.0.9',
    description='2017 OOAO DRY BUZZWORDS FOR DJANGO 2.0 with Material design',
    author='James Pic',
    author_email='jamespic@gmail.com',
    url='https://github.com/jpic/crudlfap',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    long_description=read('README.rst'),
    license='MIT',
    keywords='django crud',
    install_requires=[
        'jinja2',
        'django-jinja',
        'django-bootstrap3',
    ],
    extras_require=dict(
        django=['django'],
        tables2=['django-tables2'],
        filter=['django-filter'],
        dal=['django-autocomplete-light'],
        reversion=['django-reversion'],
        debug=['django-debug-toolbar'],
    ),
    entry_points = {
        'console_scripts': [
            'crudlfap = crudlfap_example.manage:main',
        ],
    },
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



"""
Django settings for crudlfap_example project.

Generated by 'django-admin startproject' using Django 1.9.10.

For more information on this file, see
https://docs.djangoproject.com/en/1.9/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.9/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.9/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = ')2m96!l47(!3)pvl#2@7wjh9&frxtq89*lrg(u(45gt(mnyi6d'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'debug_toolbar',

    # CRUDLFA+ dependencies
    'crudlfap',
    'bootstrap3',

    # CRUDLFA+ optionnal dependencies
    'crudlfap_filtertables2',
    'django_filters',
    'django_tables2',
    'dal',
    'dal_select2',

    # CRUDLFA+ examples
    'crudlfap_example.artist',
    'crudlfap_example.song',
]

MIDDLEWARE_CLASSES = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'debug_toolbar.middleware.DebugToolbarMiddleware',
]
INTERNAL_IPS = ('127.0.0.1',)

ROOT_URLCONF = 'crudlfap_example.urls'

TEMPLATES = [
    {
        "BACKEND": "django_jinja.backend.Jinja2",
        "APP_DIRS": True,
        "OPTIONS": {
            "app_dirname": "jinja2",
            "match_extension": ".html",
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.i18n",
                "crudlfap.context_processors.base",
            ],
            "extensions": [
                "jinja2.ext.do",
                "jinja2.ext.loopcontrols",
                "jinja2.ext.with_",
                "jinja2.ext.i18n",
                "jinja2.ext.autoescape",
                "django_jinja.builtins.extensions.CsrfExtension",
                "django_jinja.builtins.extensions.CacheExtension",
                "django_jinja.builtins.extensions.TimezoneExtension",
                "django_jinja.builtins.extensions.UrlsExtension",
                "django_jinja.builtins.extensions.StaticFilesExtension",
                "django_jinja.builtins.extensions.DjangoFiltersExtension",
            ],
            "constants": {
                "settings": dict(
                    SITE_NAME='CRUFLA+DEMO',
                    SITE_TITLE='CRUFLA+ demo !',
                ),
            },
            "globals": {
                "pagination_filter_params": "crudlfap.jinja2.pagination_filter_params",  # noqa
                "Router": "crudlfap.routers.Router",
                "getattr": getattr,
                "isinstance": isinstance,
                "render_table": "crudlfap_filtertables2.jinja2.render_table",
                "render_form": "bootstrap3.forms.render_form",

                # For django-bootsrap4, not used but kept for reference
                "bootstrap_setting": "bootstrap4.bootstrap.get_bootstrap_setting",  # noqa
                "bootstrap_form": "bootstrap4.forms.render_form",
                "bootstrap_button": "bootstrap4.forms.render_button",
            },
            "newstyle_gettext": True,
            "bytecode_cache": {
                "name": "default",
                "backend": "django_jinja.cache.BytecodeCache",
                "enabled": False,
            },
            "autoescape": True,
            "auto_reload": DEBUG,
            "translation_engine": "django.utils.translation",
        }
    },
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]
LOGIN_REDIRECT_URL = '/'
LOCALE_PATHS = (
    os.path.join(BASE_DIR, "locale"),
)

WSGI_APPLICATION = 'crudlfap_example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/1.9/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',  # noqa
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',  # noqa
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',  # noqa
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',  # noqa
    },
]


# Internationalization
# https://docs.djangoproject.com/en/1.9/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.9/howto/static-files/

STATIC_URL = '/static/'
